"""
Heavily based on handler.py from graypy.
"""
import logging
from logging.handlers import DatagramHandler, SysLogHandler
import os.path
import sys
import socket

from influxpy.protocol import encode_line

NANOSECONDS_PER_SECOND = 1000 * 1000 * 1000
SYSLOG_LEVELS = {
    logging.CRITICAL: SysLogHandler.LOG_CRIT,
    logging.ERROR: SysLogHandler.LOG_ERR,
    logging.WARNING: SysLogHandler.LOG_WARNING,
    logging.INFO: SysLogHandler.LOG_INFO,
    logging.DEBUG: SysLogHandler.LOG_DEBUG,
}


class UDPHandler(DatagramHandler):
    """
    Log messages to InfluxDB using UDP.

    :param host: The host of the InfluxDB server.
    :param port: The port of the InfluxDB server.
    :param measurement: The name of the "measurement" (table) in InfluxDB.
    :param debugging_fields: Send debug fields if true (the default).
    :param extra_fields: Send extra fields on the log record to InfluxDB
        if true (the default).
    :param localname: Use specified hostname as host tag instead
        of socket.gethostname()
    :param facility: Set facility tag to use, else some magic based
        on sys.argv[0] happens.
    :param sock: for unit testing
    """
    def __init__(self, host, port, measurement, debugging_fields=True,
                 extra_fields=True, localname=None, fqdn=False, facility=None,
                 sock=None):
        DatagramHandler.__init__(self, host, int(port))
        self.measurement = measurement
        self.debugging_fields = debugging_fields
        self.extra_fields = extra_fields
        self.fqdn = fqdn
        self.localname = localname
        self.facility = facility
        self.sock = sock

        # Do some magic based on sys.argv[0] to figure out some kind of
        # facility - it might be better to just let the user decide...
        if not self.facility:
            self.facility = os.path.basename(sys.argv[0])
            self.facility = self.facility.replace(".", "_").replace("-", "_")
            self.facility = self.facility.replace(" ", "_")
            self.facility = self.facility.strip("_").lower()
            if not self.facility:
                raise ValueError("facility magic failed")

    def makePickle(self, record):
        """
        Prepare all info and use influx.protocol.encode_line to create
        the message.
        """
        ts = int(record.created * NANOSECONDS_PER_SECOND)
        msg = record.getMessage()
        if self.localname:
            host = self.localname
        else:
            host = socket.getfqdn() if self.fqdn else socket.gethostname()

        tags = {
            "host": host,
            "level": SYSLOG_LEVELS.get(record.levelno, record.levelno),
            "level_name": logging.getLevelName(record.levelno),
            "facility": self.facility,
            "logger": record.name,
        }
        fields = {
            "message": msg,
        }

        # Use pre-formatted exception information in cases where the primary
        # exception information was removed, eg. for LogRecord serialization
        if record.exc_info and not record.exc_text:
            # format exception information if present
            formatter = logging._defaultFormatter
            record.exc_text = formatter.formatException(record.exc_info)
        if record.exc_text:
            fields["full_message"] = "\n".join([msg, record.exc_text])

        # Add debugging fields if enabled
        if self.debugging_fields:
            fields.update({
                "file": record.pathname,
                "line": record.lineno,
                "function": record.funcName,
                "pid": record.process,
                "thread_name": record.threadName,
            })
            pn = getattr(record, "processName", None)
            if pn is not None:
                fields["process_name"] = pn

        if self.extra_fields:
            fields = add_extra_fields(fields, record)

        return encode_line(self.measurement, tags, fields, ts=ts)


def add_extra_fields(d, record):
    """
    Put extra key/values found in record.__dict__ into d.

    :param d: dict for values
    :param record: The log record.
    """
    # skip_list is used to filter additional fields in a log message.
    # It contains all attributes listed in
    # http://docs.python.org/library/logging.html#logrecord-attributes
    # plus exc_text, which is only found in the logging module source,
    # and id, which is prohibited by the GELF format.
    skip_list = (
        "args", "asctime", "created", "exc_info",  "exc_text", "filename",
        "funcName", "id", "levelname", "levelno", "lineno", "module",
        "msecs", "message", "msg", "name", "pathname", "process",
        "processName", "relativeCreated", "thread", "threadName"
    )
    for key, value in record.__dict__.items():
        if value is None:
            continue
        if key not in skip_list and not key.startswith("_"):
            d[key] = value
    return d



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name="influxpy",
    version="0.1.1",
    description="Python logging handler that sends messages to InfluxDB using the line protocol over UDP.",
    long_description=open("README.md").read(),
    keywords="influxdb grafana logger logging udp",
    author="Arne Welzel",
    author_email="arne.welzel@gmail.com",
    url="https://github.com/awelzel/influxpy",
    license="BSD License",
    packages=find_packages(),
    classifiers=["License :: OSI Approved :: BSD License",
                 "Programming Language :: Python :: 2",
                 "Programming Language :: Python :: 3"],
)



"""
Quick shot at the line protocol encoding of InfluxDB.
"""
import sys
if sys.version_info[0] == 3:
    int_types = (int,)
else:
    int_types = (int, long)


__all__ = ["encode_line"]


def encode_line(measurement, tags, fields, ts=None):
    """
    Given measurement, tags and fields create a valid influxdb line.

    https://docs.influxdata.com/influxdb/v1.6/write_protocols/line_protocol_tutorial/

    :param measurement: name of the measurement
    :param tags: Tags to be used, can be empty
    :param fields: Tags to be used. For no tags, set to None, False, {}, etc.
    :param ts: Set the timestamp explicitly as int in nanoseconds. If not
        provided the server will use its current time.
    """
    if not measurement:
        raise ValueError("none or empty measurement")
    if len(fields.keys()) < 1:
        raise ValueError("need at least one field value.")
    if ts is not None:
        try:
            int(str(ts + 1))
        except (ValueError, TypeError):
            raise ValueError("ts not an integer ({})".format(type(ts)))

    tags = tags or {}
    tags = _encode_tags(tags)
    fields = _encode_fields(fields)
    ts = " {:d}".format(ts) if ts else ""
    line = "{:s}{:s} {:s}{:s}".format(measurement, tags, fields, ts)
    return line.encode("utf-8")


def _escape_key(k):
    k = k.replace("\\", "\\\\")
    k = k.replace(",", "\\,")
    k = k.replace("=", "\\=")
    return k.replace(" ", "\\ ")


def _encode_tags(tags):
    if not tags:
        return ""
    escaped = []
    for k in sorted(tags):
        v = tags[k]
        v = _escape_key(str(tags[k]))
        k = _escape_key(str(k))
        escaped.append((k, v))
    return ",".join([""] + ["{:s}={:s}".format(k, v) for k, v in escaped])


def _escape_field_value(s):
    s = s.replace("\\", "\\\\")
    return s.replace('"', '\\"')


def _encode_fields(values):
    escaped = []
    for k in sorted(values):
        v = values[k]
        # bool first: isinstance(True, int) is True
        if isinstance(v, bool):
            v = "true" if v else "false"
        elif isinstance(v, int_types):
            v = "{:d}i".format(v)  # send as an actual integer
        elif isinstance(v, float):
            v = str(v)
        else:
            # Assume it is some sort of string
            v = '"{}"'.format(_escape_field_value(str(v)))
        escaped.append((_escape_key(k), v))

    return ",".join(["=".join([k, v]) for k, v in escaped])
